<?php
declare(strict_types=1);

namespace App\Controller;

use Cake\Controller\Controller;
use Cake\Core\Configure;
use Cake\Datasource\Exception\RecordNotFoundException;

/**
 * Application Controller
 *
 * Add your application-wide methods in the class below, your controllers
 * will inherit them.
 *
 * @link https://book.cakephp.org/4/en/controllers.html#the-app-controller
 */
class AppController extends Controller
{
    const MSG = [
        'UnauthorizedIP' => '許可されていないIPです。'
    ];

    /**
     * Initialization hook method.
     *
     * Use this method to add common initialization code like loading components.
     *
     * e.g. `$this->loadComponent('FormProtection');`
     *
     * @return void
     */
    public function initialize(): void
    {
        parent::initialize();

        $this->loadComponent('RequestHandler');
        $this->loadComponent('Flash');

        /*
         * Enable the following component for recommended CakePHP form protection settings.
         * see https://book.cakephp.org/4/en/controllers/components/form-protection.html
         */
        $this->loadComponent('FormProtection');

        if (! array_key_exists($this->request->getParam('controller'), Configure::read('NotIpCheckController'))) {
            // 特定コントローラー以外はIPチェック
            if (Configure::read('CustomSetting.env') === 'product') {
                $this->_ipCheck(env('HTTP_X_FORWARDED_FOR')); // 本番環境はプロキシ挟むのでXFFで判定
            } else {
                $this->_ipCheck($this->request->clientIp());
            }
        }
    }

    private function _ipCheck($ip) {
        try {
            $this->fetchTable('AuthIp')->findByIpAndEnabled($ip, 1)->firstOrFail();
        } catch (RecordNotFoundException $e) {
            return $this->redirect(['controller' => 'Err', 'action' => 'index', self::MSG['UnauthorizedIP']]);
        }
    }
}
